==> exonmap_2.0.03.tar.gz: exonmap/R/mapping.R <==
.xmap <- function(name,v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction"),probes.min) {
  
  if(length(v) == 0) { return(NULL) }

  if(length(subset) > 1) subset <- "core"
  subset  <- match.arg(subset)

  #have to treat prediction stuff as a separate case
  if(subset == "prediction") { .xmap.pred(name,v,as.vector,unique,subset,probes.min) }
  else                       { .xmap.core.est(name,v,as.vector,unique,subset,probes.min) }
}


.xmap.core.est <- function(name,v,as.vector,unique,subset,probes.min) {
  cl  <- .xmap.call.map(name,subset)
  sql <- as.character(cl[1,2])
  con <- .xmap.get.con(as.character(cl[1,4]))
  

  if(is.null(con)) {
    stop(paste("Can't find database subset for '",subset,"' mapping. Is the config file 'databases.txt' set up correctly?",sep=""))
  }

  v   <- .make.params(v)
  r   <- lapply(v,.xmap.call,sql=sql,con=con)
  r   <- do.call(rbind,r)

  if(!is.null(r)) {
    if(unique) {
      r <- unique(r)
    }
  
    if(!missing(probes.min)) {
      cns <- colnames(r)
      if(any(cns == "num_probes")) {
        idx <- "num_probes"
        r <- r[r[, idx] >= probes.min,,drop=FALSE]
      }
      else if(any(cns == "probe_count")) {
        idx <- "probe_count"
        r <- r[r[, idx] >= probes.min,,drop=FALSE]
      }
      else {
          #do nothing
      }
    }
    if(as.vector) {
      r <- unlist(r[,cl[1,3]])
      if(unique) {
        r <- unique(r)
      }
    }
  }
  r
}

.xmap.pred <- function(name,v,as.vector,unique,subset,probes.min) {

  cl  <- .xmap.call.map(name,subset)
  sql <- as.character(cl[1,2])
  con <- .xmap.get.con(as.character(cl[1,4]))




  if(is.null(con)) {
    stop(paste("Can't find database subset for '",subset,"' mapping. Is the config file 'databases.txt' set up correctly?",sep=""))
  }

  #Exons and transcripts from the prediction subset are named by integers,
  #not strings - so can't generate a comma separated list for the database
  #Must do them 1 by 1 
  if(any(name==c("ps.to.e","ps.to.t"))) {
    v   <- .make.params(v)
    r   <- lapply(v,.xmap.call,sql=sql,con=con)
    r   <- do.call(rbind,r)
  }
  else {
    r   <-as.data.frame(t(sapply(v,.xmap.call,sql=sql,con=con,is.integer=TRUE)))
  }

  if(unique) {
    r <- unique(r)
  }
  
  if(!missing(probes.min)) {
    cns <- colnames(r)
    if(any(cns == "num_probes")) {
      idx <- "num_probes"
      r <- r[r[, idx] >= probes.min,,drop=FALSE]
    }
    else if(any(cns == "probe_count")) {
      idx <- "probe_count"
      r <- r[r[, idx] >= probes.min,,drop=FALSE]
    }
    else {
                                        #do nothing
    }
  }
  if(as.vector) {
    r <- unlist(r[,cl[1,3]])
    if(unique) {
      r <- unique(r)
    }
  }
  r
}


probeset.to.probe <- function(v,as.vector=TRUE,unique=TRUE,mt.rm=TRUE) {
  v <- multitarget(v,exclude=mt.rm)
  if(length(v) == 0) return(v)
  .xmap("ps.to.p",v,as.vector,unique)
}

probeset.to.exon <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction")) {
  .xmap("ps.to.e",v,as.vector,unique,subset)
}

probeset.to.transcript <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction")) {
  .xmap("ps.to.t",v,as.vector,unique,subset)
}

probeset.to.gene <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est")) {
  .xmap("ps.to.g",v,as.vector,unique,subset)
}

exon.to.probeset <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction"),probes.min=4) {
  .xmap("e.to.ps",v,as.vector,unique,subset,probes.min)
}

exon.to.transcript <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction")) {
  .xmap("e.to.t",v,as.vector,unique,subset)
}

exon.to.gene <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est")) {
  .xmap("e.to.g",v,as.vector,unique,subset)
}

transcript.to.probeset <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction"),probes.min=4) {
  .xmap("t.to.ps",v,as.vector,unique,subset,probes.min)
}

transcript.to.exon <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction")) {
  .xmap("t.to.e",v,as.vector,unique,subset)
}

transcript.to.gene <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est")) {
  .xmap("t.to.g",v,as.vector,unique,subset)
}

gene.to.probeset <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est"),probes.min=4) {
  .xmap("g.to.ps",v,as.vector,unique,subset,probes.min)
}

gene.to.exon <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est")) {
  .xmap("g.to.e",v,as.vector,unique,subset)
}

gene.to.transcript <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est")) {
  .xmap("g.to.t",v,as.vector,unique,subset)
}

symbol.to.gene <- function(v,as.vector=TRUE,unique=TRUE) {
  if(length(v) == 0) { return(NULL) }
  species <- .xmap.get.species("core")
  con     <- .xmap.get.con("core")

  # num.spe IS IGNORED in v.50 and above
  # - IT WILL DISAPPEAR FROM FUTURE VERSIONS OF X:Map
  # - (IT IS HERE FOR COMPATIBILITY)

  if(species=="homo_sapiens")  num.spe <- 1100
  if(species=="mus_musculus")  num.spe <- 1400
  if(species=="rattus_norvegicus") num.spe <- 2800

  v   <- .make.params(v)
  sql <- paste("CALL xmap_externalDbidToGene(",num.spe,",'",v,"')",sep="")
  r <- .xmap.raw.call(sql,con)
  if(!is.null(r) & (dim(r)[2] >= 3)) {
    if(unique) {
   	r <- unique(r)
    }
   
    if(as.vector) {
      r <- r[,3]
    }
  }
  r
}

gene.to.exon.probeset <- function(v,unique=TRUE,probes.min=4) {
  if(length(v) == 0) { return(NULL) }
  con <- .xmap.get.con("core")
  sql <- "xmap_geneToExonProbeset"
  r   <- lapply(v,.xmap.call,sql=sql,con=con)
  r   <- do.call(rbind,r)
  if(!is.null(r) & (dim(r)[2] > 0)) {
    if(any(colnames(r) == "probe_count")) { idx <- "probe_count" }
    else                                  { idx <- "num_probes" }
    r <- r[r[,idx] >= probes.min,,drop=FALSE]
    if(unique) {
      r <- unique(r)
    }
  }
  r
}

gene.to.exon.probeset.expr <- function(x,v,probes.min=4) {
  if(length(v) == 0) { return(NULL) }
  if(class(x) == "ExpressionSet") { x <- exprs(x) }
  r <- gene.to.exon.probeset(v,probes.min=probes.min)
  ps.r <- r[,"probeset_name"]
  ps   <- intersect(ps.r,rownames(x))
  
  r <- r[ps.r %in% ps,,drop=FALSE]
 
  x <- as.data.frame(x[rownames(x) %in% ps,,drop=FALSE])
  x <- cbind(probeset_name=rownames(x),x)
  r <- merge(x,r,by="probeset_name")
  #move probeset_name column
  idx <- c((2:length(colnames(x))), 1, (dim(x)[2] + 1):length(r))
  r <- r[,idx,drop=FALSE]
  r
}

exon.to.sequence <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est")) {
  .xmap("e.to.s",v,as.vector,unique,subset)
}

==> exonmap_2.8.0.tar.gz: exonmap/R/mapping.R <==
.xmap <- function(name,v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction"),probes.min) {
  
  if(length(v) == 0) { return(NULL) }

  if(length(subset) > 1) subset <- "core"
  subset  <- match.arg(subset)

  #have to treat prediction stuff as a separate case
  if(subset == "prediction") { .xmap.pred(name,v,as.vector,unique,subset,probes.min) }
  else                       { .xmap.core.est(name,v,as.vector,unique,subset,probes.min) }
}


.xmap.core.est <- function(name,v,as.vector,unique,subset,probes.min) {
  cl  <- .xmap.call.map(name,subset)
  sql <- as.character(cl[1,2])
  con <- .xmap.get.con(as.character(cl[1,4]))
  

  if(is.null(con)) {
    stop(paste("Can't find database subset for '",subset,"' mapping. Is the config file 'databases.txt' set up correctly?",sep=""))
  }

  v   <- .make.params(v)
  r   <- lapply(v,.xmap.call,sql=sql,con=con)
  r   <- do.call(rbind,r)

  if(!is.null(r)) {
    if(unique) {
      r <- unique(r)
    }
  
    if(!missing(probes.min)) {
      cns <- colnames(r)
      if(any(cns == "num_probes")) {
        idx <- "num_probes"
        r <- r[r[, idx] >= probes.min,,drop=FALSE]
      }
      else if(any(cns == "probe_count")) {
        idx <- "probe_count"
        r <- r[r[, idx] >= probes.min,,drop=FALSE]
      }
      else {
          #do nothing
      }
    }
    if(as.vector) {
      r <- unlist(r[,cl[1,3]])
      if(unique) {
        r <- unique(r)
      }
    }
  }
  r
}

.xmap.pred <- function(name,v,as.vector,unique,subset,probes.min) {

  cl  <- .xmap.call.map(name,subset)
  sql <- as.character(cl[1,2])
  con <- .xmap.get.con(as.character(cl[1,4]))




  if(is.null(con)) {
    stop(paste("Can't find database subset for '",subset,"' mapping. Is the config file 'databases.txt' set up correctly?",sep=""))
  }

  #Exons and transcripts from the prediction subset are named by integers,
  #not strings - so can't generate a comma separated list for the database
  #Must do them 1 by 1 
  if(any(name==c("ps.to.e","ps.to.t"))) {
    v   <- .make.params(v)
    r   <- lapply(v,.xmap.call,sql=sql,con=con)
    r   <- do.call(rbind,r)
  }
  else {
    r   <-as.data.frame(t(sapply(v,.xmap.call,sql=sql,con=con,is.integer=TRUE)))
  }

  if(unique) {
    r <- unique(r)
  }
  
  if(!missing(probes.min)) {
    cns <- colnames(r)
    if(any(cns == "num_probes")) {
      idx <- "num_probes"
      r <- r[r[, idx] >= probes.min,,drop=FALSE]
    }
    else if(any(cns == "probe_count")) {
      idx <- "probe_count"
      r <- r[r[, idx] >= probes.min,,drop=FALSE]
    }
    else {
                                        #do nothing
    }
  }
  if(as.vector) {
    r <- unlist(r[,cl[1,3]])
    if(unique) {
      r <- unique(r)
    }
  }
  r
}


probeset.to.probe <- function(v,as.vector=TRUE,unique=TRUE,mt.rm=TRUE) {
  v <- multitarget(v,exclude=mt.rm)
  if(length(v) == 0) return(v)
  .xmap("ps.to.p",v,as.vector,unique)
}

probeset.to.exon <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction")) {
  .xmap("ps.to.e",v,as.vector,unique,subset)
}

probeset.to.transcript <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction")) {
  .xmap("ps.to.t",v,as.vector,unique,subset)
}

probeset.to.gene <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est")) {
  .xmap("ps.to.g",v,as.vector,unique,subset)
}

exon.to.probeset <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction"),probes.min=4) {
  .xmap("e.to.ps",v,as.vector,unique,subset,probes.min)
}

exon.to.transcript <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction")) {
  .xmap("e.to.t",v,as.vector,unique,subset)
}

exon.to.gene <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est")) {
  .xmap("e.to.g",v,as.vector,unique,subset)
}

transcript.to.probeset <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction"),probes.min=4) {
  .xmap("t.to.ps",v,as.vector,unique,subset,probes.min)
}

transcript.to.exon <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est","prediction")) {
  .xmap("t.to.e",v,as.vector,unique,subset)
}

transcript.to.gene <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est")) {
  .xmap("t.to.g",v,as.vector,unique,subset)
}

gene.to.probeset <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est"),probes.min=4) {
  .xmap("g.to.ps",v,as.vector,unique,subset,probes.min)
}

gene.to.exon <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est")) {
  .xmap("g.to.e",v,as.vector,unique,subset)
}

gene.to.transcript <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est")) {
  .xmap("g.to.t",v,as.vector,unique,subset)
}

symbol.to.gene <- function(v,as.vector=TRUE,unique=TRUE) {
  if(length(v) == 0) { return(NULL) }
  species <- .xmap.get.species("core")
  con     <- .xmap.get.con("core")

  # num.spe IS IGNORED in v.50 and above
  # - IT WILL DISAPPEAR FROM FUTURE VERSIONS OF X:Map
  # - (IT IS HERE FOR COMPATIBILITY)

  if(species=="homo_sapiens")  num.spe <- 1100
  if(species=="mus_musculus")  num.spe <- 1400
  if(species=="rattus_norvegicus") num.spe <- 2800

  v   <- .make.params(v)
  sql <- paste("CALL xmap_externalDbidToGene(",num.spe,",'",v,"')",sep="")
  r <- .xmap.raw.call(sql,con)
  if(!is.null(r) & (dim(r)[2] >= 3)) {
    if(unique) {
   	r <- unique(r)
    }
   
    if(as.vector) {
      r <- r[,3]
    }
  }
  r
}

gene.to.exon.probeset <- function(v,unique=TRUE,probes.min=4,mt.rm=TRUE) {
  if(length(v) == 0) { return(NULL) }
  con <- .xmap.get.con("core")
  sql <- if( mt.rm ) "xmap_geneToExonProbeset" else "xmap_geneToExonProbesetMT"
  r   <- lapply(v,.xmap.call,sql=sql,con=con)
  r   <- do.call(rbind,r)
  if(!is.null(r) & (dim(r)[2] > 0)) {
    if(any(colnames(r) == "probe_count")) { idx <- "probe_count" }
    else                                  { idx <- "num_probes" }
    r <- r[r[,idx] >= probes.min,,drop=FALSE]
    if(unique) {
      r <- unique(r)
    }
  }
  r
}

gene.to.exon.probeset.expr <- function(x,v,probes.min=4,mt.rm=TRUE) {
  if(length(v) == 0) { return(NULL) }
  if(class(x) == "ExpressionSet") { x <- exprs(x) }
  r <- gene.to.exon.probeset(v,probes.min=probes.min,mt.rm=mt.rm)
  ps.r <- r[,"probeset_name"]
  ps   <- intersect(ps.r,rownames(x))
  
  r <- r[ps.r %in% ps,,drop=FALSE]
 
  x <- as.data.frame(x[rownames(x) %in% ps,,drop=FALSE])
  x <- cbind(probeset_name=rownames(x),x)
  r <- merge(x,r,by="probeset_name")
  #move probeset_name column
  idx <- c((2:length(colnames(x))), 1, (dim(x)[2] + 1):length(r))
  r <- r[,idx,drop=FALSE]
  r
}

exon.to.sequence <- function(v,as.vector=TRUE,unique=TRUE,subset=c("core","est")) {
  .xmap("e.to.s",v,as.vector,unique,subset)
}
